@@ -102,6 +102,8 @@
 /*	Patrik Rak
 /*	Modra 6
 /*	155 00, Prague, Czech Republic
+/*
+/*	Wietse Venema
 /*--*/
 
 /* System library. */
@@ -505,9 +507,15 @@ static int qmgr_message_read(QMGR_MESSAGE *message)
 			message->rflags |= QMGR_READ_FLAG_SEEN_ALL_NON_RCPT;
 			break;
 		    }
-		    /* Examine non-recipient records in extracted segment. */
-		    if (vstream_fseek(message->fp, message->data_offset
-				      + message->data_size, SEEK_SET) < 0)
+
+		    /*
+		     * Examine non-recipient records in the extracted
+		     * segment. Note that this skips to the message start
+		     * record, because the handler for that record changes
+		     * the expectations for allowed record types.
+		     */
+		    if (vstream_fseek(message->fp, message->data_offset,
+				      SEEK_SET) < 0)
 			msg_fatal("seek file %s: %m", VSTREAM_PATH(message->fp));
 		    continue;
 		}
@@ -1201,7 +1209,9 @@ static void qmgr_message_resolve(QMGR_MESSAGE *message)
 	 */
 	if (*var_defer_xports && (message->qflags & QMGR_FLUSH_DFXP) == 0) {
 	    if (defer_xport_argv == 0)
-		defer_xport_argv = argv_split(var_defer_xports, CHARS_COMMA_SP);
+		defer_xport_argv = argv_split_cw(var_defer_xports,
+						 CHARS_COMMA_SP,
+						 VAR_DEFER_XPORTS);
 	    for (cpp = defer_xport_argv->argv; *cpp; cpp++)
 		if (strcmp(*cpp, STR(reply.transport)) == 0)
 		    break;
