@@ -80,6 +80,8 @@
 /*	New York, NY 10011, USA
 /*
 /*	Viktor Dukhovni
+/*
+/*	Wietse Venema
 /*--*/
 
 /* System library. */
@@ -250,7 +252,7 @@ static void tls_policy_lookup_one(SMTP_TLS_POLICY *tls, int *site_level,
     }
     saved_policy = policy = mystrdup(lookup);
 
-    if ((tok = mystrtok(&policy, CHARS_COMMA_SP)) == 0) {
+    if ((tok = mystrtok_cw(&policy, CHARS_COMMA_SP, tls_policy->title)) == 0) {
 	msg_warn("%s: invalid empty policy", WHERE);
 	INVALID_RETURN(tls->why, site_level);
     }
@@ -265,7 +267,7 @@ static void tls_policy_lookup_one(SMTP_TLS_POLICY *tls, int *site_level,
      * Warn about ignored attributes when TLS is disabled.
      */
     if (*site_level < TLS_LEV_MAY) {
-	while ((tok = mystrtok(&policy, CHARS_COMMA_SP)) != 0)
+	while ((tok = mystrtok_cw(&policy, CHARS_COMMA_SP, tls_policy->title)) != 0)
 	    msg_warn("%s: ignoring attribute \"%s\" with TLS disabled",
 		     WHERE, tok);
 	FREE_RETURN;
@@ -275,7 +277,7 @@ static void tls_policy_lookup_one(SMTP_TLS_POLICY *tls, int *site_level,
      * Errors in attributes may have security consequences, don't ignore
      * errors that can degrade security.
      */
-    while ((tok = mystrtok(&policy, CHARS_COMMA_SP)) != 0) {
+    while ((tok = mystrtok_cw(&policy, CHARS_COMMA_SP, tls_policy->title)) != 0) {
 	if ((err = split_nameval(tok, &name, &val)) != 0) {
 	    msg_warn("%s: malformed attribute/value pair \"%s\": %s",
 		     WHERE, tok, err);
@@ -336,8 +338,7 @@ static void tls_policy_lookup_one(SMTP_TLS_POLICY *tls, int *site_level,
 	    case TLS_LEV_FPRINT:
 		if (!tls->dane)
 		    tls->dane = tls_dane_alloc();
-		tls_dane_add_ee_digests(tls->dane, var_smtp_tls_fpt_dgst,
-					val, "|", smtp_mode);
+		tls_dane_add_fpt_digests(tls->dane, val, "|", smtp_mode);
 		break;
 	    case TLS_LEV_VERIFY:
 	    case TLS_LEV_SECURE:
@@ -426,7 +427,7 @@ static void tls_policy_lookup(SMTP_TLS_POLICY *tls, int *site_level,
 
 /* load_tas - load one or more ta files */
 
-static int load_tas(TLS_DANE *dane, const char *files)
+static int load_tas(TLS_DANE *dane, const char *files, const char *blame)
 {
     int     ret = 0;
     char   *save = mystrdup(files);
@@ -434,7 +435,7 @@ static int load_tas(TLS_DANE *dane, const char *files)
     char   *file;
 
     do {
-	if ((file = mystrtok(&buf, CHARS_COMMA_SP)) != 0)
+	if ((file = mystrtok_cw(&buf, CHARS_COMMA_SP, blame)) != 0)
 	    ret = tls_dane_load_trustfile(dane, file);
     } while (file && ret);
 
@@ -618,11 +619,10 @@ static void *policy_create(const char *unused_key, void *context)
     case TLS_LEV_FPRINT:
 	if (tls->dane == 0)
 	    tls->dane = tls_dane_alloc();
-	if (!TLS_DANE_HASEE(tls->dane)) {
-	    tls_dane_add_ee_digests(tls->dane, var_smtp_tls_fpt_dgst,
-				    var_smtp_tls_fpt_cmatch, CHARS_COMMA_SP,
-				    smtp_mode);
-	    if (!TLS_DANE_HASEE(tls->dane)) {
+	if (tls->dane->tlsa == 0) {
+	    tls_dane_add_fpt_digests(tls->dane, var_smtp_tls_fpt_cmatch,
+				     CHARS_COMMA_SP, smtp_mode);
+	    if (tls->dane->tlsa == 0) {
 		msg_warn("nexthop domain %s: configured at fingerprint "
 		       "security level, but with no fingerprints to match.",
 			 dest);
@@ -635,14 +635,16 @@ static void *policy_create(const char *unused_key, void *context)
     case TLS_LEV_SECURE:
 	if (tls->matchargv == 0)
 	    tls->matchargv =
-		argv_split(tls->level == TLS_LEV_VERIFY ?
-			   var_smtp_tls_vfy_cmatch : var_smtp_tls_sec_cmatch,
-			   CHARS_COMMA_SP ":");
+		argv_split_cw(tls->level == TLS_LEV_VERIFY ?
+			  var_smtp_tls_vfy_cmatch : var_smtp_tls_sec_cmatch,
+			  CHARS_COMMA_SP ":", tls->level == TLS_LEV_VERIFY ?
+			VAR_SMTP_TLS_VFY_CMATCH : VAR_SMTP_TLS_SEC_CMATCH);
 	if (*var_smtp_tls_tafile) {
 	    if (tls->dane == 0)
 		tls->dane = tls_dane_alloc();
-	    if (!TLS_DANE_HASTA(tls->dane)
-		&& !load_tas(tls->dane, var_smtp_tls_tafile)) {
+	    if (tls->dane->tlsa == 0
+		&& !load_tas(tls->dane, var_smtp_tls_tafile,
+			     VAR_SMTP_TLS_TAFILE)) {
 		MARK_INVALID(tls->why, &tls->level);
 		return ((void *) tls);
 	    }
@@ -910,7 +912,7 @@ static void dane_init(SMTP_TLS_POLICY *tls, SMTP_ITERATOR *iter)
     /*
      * With DANE trust anchors, peername matching is not configurable.
      */
-    if (TLS_DANE_HASTA(dane)) {
+    if (dane->tlsa != 0) {
 	tls->matchargv = argv_alloc(2);
 	argv_add(tls->matchargv, dane->base_domain, ARGV_END);
 	if (iter->mx) {
@@ -920,7 +922,7 @@ static void dane_init(SMTP_TLS_POLICY *tls, SMTP_ITERATOR *iter)
 		argv_add(tls->matchargv, iter->mx->rname,
 			 iter->mx->qname, ARGV_END);
 	}
-    } else if (!TLS_DANE_HASEE(dane))
+    } else
 	msg_panic("empty DANE match list");
     tls->dane = dane;
     return;
