@@ -79,6 +79,8 @@
 /*	Google, Inc.
 /*	111 8th Avenue
 /*	New York, NY 10011, USA
+/*
+/*	Wietse Venema
 /*--*/
 
 /* System library. */
@@ -319,7 +321,7 @@ static void load_pkey(pem_load_state_t *st, int pkey_type,
 	if (pkey)
 	    EVP_PKEY_free(pkey);
 
-	/* XXX: Legacy behaviour was silent, should we stay silent? */
+	/* XXX: Legacy behavior was silent, should we stay silent? */
 	if (st->mixed) {
 	    msg_warn("ignoring 2nd key at index %d in %s after 1st at %d",
 		     st->objnum, st->source, st->keynum);
@@ -369,7 +371,7 @@ static int load_pem_object(pem_load_state_t *st)
 		   && strcmp(name, PEM_STRING_DSA) == 0)) {
 	load_pkey(st, pkey_type, buf, buflen);
     } else if (!st->mixed) {
-       msg_warn("loading %s: ignoring PEM type: %s", st->source, name);
+	msg_warn("loading %s: ignoring PEM type: %s", st->source, name);
     }
     OPENSSL_free(name);
     OPENSSL_free(header);
@@ -460,7 +462,9 @@ static int load_pem_bio(pem_load_state_t *st, int more)
 static int load_chain_files(SSL_CTX *ctx, const char *chain_files)
 {
     pem_load_state_t st;
-    ARGV   *files = argv_split(chain_files, CHARS_COMMA_SP);
+
+    /* XXX Which parameter to blame? */
+    ARGV   *files = argv_split_cw(chain_files, CHARS_COMMA_SP, "chain_files");
     char  **filep;
     int     ret = 0;
     int     more;
